﻿using System;
using System.Drawing;

namespace lcdST7735com
{
	class ImageCtrl
	{
		public int ImgW { get; private set; }
		public int ImgH { get; private set; }
		public bool IsError { get; private set; }
		private Bitmap img;

		//コンストラクタ
		public ImageCtrl(string filename)
		{
			try
			{
				img = new Bitmap(filename);
				ImgW = img.Width;
				ImgH = img.Height;
				IsError = false;
			}
			catch { img = null; ImgW = ImgH = 0; IsError = true; }
		}

		//縦横サイズをリサイズする
		//引数:	(width=n, height=0) または (width=0, height=n) または (width=m, height=n)
		public void Resize(int width, int height)
		{
			//縦横比を保ち、幅を合わせる
			if (height == 0)
			{
				height = (int)((double)img.Height * (double)width / (double)img.Width);
			}
			//縦横比を保ち、高さを合わせる
			else if (width == 0)
			{
				width = (int)((double)img.Width * (double)height / (double)img.Height);
			}
			//それ以外の場合…縦横比を無視し、幅と高さを合わせる

			try
			{
				img = new Bitmap(img, width, height);
				ImgW = img.Width;
				ImgH = img.Height;
			}
			catch { }
		}

		//全画素データを16bit色値で取得する（LCD転送用データ）
		//戻り値:	RGB=5:6:5bitの色値をint型配列で返す
		public int[] GetPixelArray()
		{
			Color[] srcColors = GetPixels();
			int[] lcdColors = ConvertToRGB565(srcColors);
			return lcdColors;
		}

		//全画素データをColor形式で取得する
		private Color[] GetPixels()
		{
			Color[] colors = new Color[img.Width * img.Height];
			for (int p = 0, y = 0; y < img.Height; y++)
				for (int x = 0; x < img.Width; x++, p++)
					colors[p] = img.GetPixel(x, y);
			return colors;
		}

		//全画素データをLCD用の色値形式に変換する
		private int[] ConvertToRGB565(Color[] colors)
		{
			int[] lcdColors = new int[colors.Length];
			for (int rr, gg, bb, i = 0; i < colors.Length; i++)
			{
				rr = colors[i].R * 31 / 255;	//2^5-1=31
				gg = colors[i].G * 63 / 255;	//2^6-1=63
				bb = colors[i].B * 31 / 255;	//2^5-1=31
				lcdColors[i] = (rr << 11) | (gg << 5) | bb;		//16bit
			}
			return lcdColors;
		}

	}	//class
}	//namespace
